@@ -31,6 +31,7 @@
 #include <dirent.h>
 #include <signal.h>
 #include <gnome.h>
+#include <glib/gstdio.h>
 #include <cspi/spi.h>
 #include <libbonobo.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -155,6 +156,8 @@ typedef struct _GokArgs {
 	gboolean list_accessmethods;
 	char* usb_vid_pid;
 	gdouble valuator_sensitivity;
+	gboolean non_system_mouse;
+	gboolean disable_autobranch;
 } GokArgs;
 
 static GokArgs gok_args;
@@ -337,6 +340,24 @@ static const GOptionEntry options[] = {
 		N_("A multiplier to be applied to input device valuator events before processing"),
 		NULL
 	},
+	{
+		"non-system-mouse", 
+		'\0', 
+		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.non_system_mouse, 
+		N_("Attempt to use GOK without the system mouse"),
+		NULL
+	},
+	{
+		"disable-autobranch",
+		0,
+		0,
+		G_OPTION_ARG_NONE,
+		&gok_args.disable_autobranch,
+		N_("Disable automatic keyboard branching"),
+		NULL
+	},
 	/* End the list */
 	{	NULL}
 };
@@ -353,9 +374,36 @@ static int gok_sig_handler(int sig)
 #if ! defined ENABLE_LOGGING_NORMAL
 		if (respawn_on_segv)
 		{
+		    GString *command = g_string_new ("gok");
+
+		    /*
+		     * Add arguments to start GOK in same state.
+		     * Perhaps other arguments should also be
+		     * supported, but this is a good start.
+		     */
+		    if (gok_args.is_login) {
+			g_string_append (command, " --login");
+		    }
+		    if (gok_args.accessmethodname) {
+			g_string_append_printf (command,
+				" --access-method=%s",
+				gok_args.accessmethodname);
+		    }
+		    if (gok_args.scanactionname) {
+			g_string_append_printf (command,
+				" --scan-action=%s",
+				gok_args.scanactionname);
+		    }
+		    if (gok_args.selectactionname) {
+			g_string_append_printf (command,
+				" --select-action=%s",
+				gok_args.selectactionname);
+		    }
+
 		    fprintf (stderr, "Restarting GOK.\n");
-		    g_on_error_stack_trace("gok");
-		    g_spawn_command_line_async ("gok", NULL);
+		    g_on_error_stack_trace ("gok");
+		    g_spawn_command_line_async (command->str, NULL);
+		    g_string_free (command, TRUE);
 		}
 #endif
 		break;
@@ -393,6 +441,8 @@ gok_args_init (GokArgs *args)
 	args->selectactionname = NULL;
 	args->usb_vid_pid = NULL;
 	args->valuator_sensitivity = 0;
+	args->non_system_mouse = 0;
+	args->disable_autobranch = FALSE;
 }
 
 /* callback for gok window state events */
@@ -566,7 +616,7 @@ gok_main_open(gint argc, gchar *argv[])
 {
 	GnomeProgram *gok_program;
 	GOptionContext *context;
-	gboolean create_dirs = TRUE;
+	gboolean create_dirs;
 	int i;
 
 	/* initialize member data */
@@ -582,17 +632,9 @@ gok_main_open(gint argc, gchar *argv[])
 	context	= g_option_context_new (_("The GNOME On-screen Keyboard"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 
-	/*
-	 * Preprocess the arguments to see if --login was included.  Need to
-	 * do this before calling gnome_program_init (which normally processes
-	 * the arguments) since this affects how we call gnome_program_init.
-	 */
-	for (i = 0; i < argc; i++) {
-		if (strcmp (argv[i], "--login") == 0) {
-			create_dirs = FALSE;
-			break;
-		}
-	}
+	/* If the user does not have a writable HOME directory, then
+	   avoid creating the directory. */
+	create_dirs = (g_access (g_get_home_dir(), W_OK) == 0);
 
 	/* Initialize Gnome */
 	gok_program = gnome_program_init (PACKAGE, VERSION,
@@ -637,8 +679,10 @@ gok_main_open(gint argc, gchar *argv[])
 	
 	    if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
 	    {
-		if (gok_args.is_login)
-		    return 1;
+		if (gok_args.is_login) {
+			g_warning ("GOK running in login mode with a failed bonobo activation");
+		    /* return 1; */
+		}
 		else {
 		    gchar *reason = NULL;
 		    if (ret == Bonobo_ACTIVATION_REG_ALREADY_ACTIVE) reason = "already active";
@@ -1065,15 +1109,19 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 {
 	GokKeyboard* pKeyboard = NULL;
 	Accessible *acc = NULL;
+	gboolean branch;
 
 	gok_log_enter();	
 
 	/* note: ref counting happens in gok-spy's listeners */
 	m_pForegroundWindowAccessible = pAccessible;
 
+	branch = !gok_main_get_disable_autobranch ();
+
 	/* reset the context menu until we receive a focus event */
 	/*gok_spy_set_context_menu_accessible (NULL, 0);*/
-	if (pAccessible && (Accessible_getRole (pAccessible) == SPI_ROLE_ALERT))
+	if (pAccessible && (Accessible_getRole (pAccessible) == SPI_ROLE_ALERT)
+	    && branch)
 	{
 	    AccessibleNode *node = g_malloc (sizeof (AccessibleNode));
 	    GokSpyUIFlags ui_flags;
@@ -1090,7 +1138,8 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 	}
 	else if (pAccessible && Accessible_getChildCount (pAccessible) == 1
 	    && (acc = Accessible_getChildAtIndex (pAccessible, 0))
-	    && Accessible_getRole (acc) == SPI_ROLE_MENU) 
+	    && Accessible_getRole (acc) == SPI_ROLE_MENU
+	    && branch)
 	{
 	    AccessibleNode *node = g_malloc (sizeof (AccessibleNode));
 	    GokSpyUIFlags menu_flags;
@@ -1524,7 +1573,11 @@ gboolean gok_main_display_scan_previous_premade ()
 	if (pKeyboard->bDynamicallyCreated != TRUE)
 	{
 		gok_log_leave();
-		return gok_main_display_scan_previous();
+
+		if (gok_main_get_disable_autobranch ())
+			return FALSE;
+		else
+			return gok_main_display_scan_previous ();
 	}
 
 	/* initialize this array */
@@ -1847,6 +1900,8 @@ GtkWidget* gok_main_create_window (struct gok_window_realize_data *window_realiz
 			     "signal::destroy", on_window1_destroy, NULL,
 			     NULL);
 
+	gtk_window_set_keep_above (GTK_WINDOW (window1), TRUE); /* please! */
+	
 	g_set_application_name (_("GOK"));
 	gtk_window_set_default_icon_name ("gok");
 
@@ -1854,7 +1909,6 @@ GtkWidget* gok_main_create_window (struct gok_window_realize_data *window_realiz
 	m_OurResizeHeight = 100;
 	gtk_window_set_default_size (GTK_WINDOW (window1), m_OurResizeWidth, m_OurResizeHeight);
 
-
 	fixed1 = gtk_fixed_new ();
 	gtk_widget_ref (fixed1);
 	gtk_object_set_data_full (GTK_OBJECT (window1), "fixed1", fixed1,
@@ -2759,7 +2813,7 @@ void gok_main_display_error (gchar* ErrorString)
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_ERROR,
 		GTK_BUTTONS_CLOSE,
-		buffer);
+		"%s", buffer);
 	
 	gtk_window_set_title (GTK_WINDOW (pDialog), _("GOK Fatal Error"));
 	gtk_dialog_run (GTK_DIALOG (pDialog));
@@ -2910,6 +2964,18 @@ gok_main_get_valuatorsensitivity_override (void)
 	return gok_args.valuator_sensitivity;
 }
 
+gboolean
+gok_main_get_use_corepointer (void) {
+	return !gok_args.non_system_mouse && !gok_args.inputdevicename;
+}
+
+gboolean
+gok_main_get_use_xevie (void) {
+	/* TODO: make xevie a user option
+	 */
+	return xevie_dpy && !gok_main_get_use_corepointer();
+}
+
 /**
 * gok_main_get_geometry
 *
@@ -2936,6 +3002,20 @@ gok_main_get_login (void)
 	return gok_args.is_login;
 }
 
+/**
+ * gok_main_get_disable_autobranch:
+ *
+ * Call this to see if the disable-autobranch argument was passed
+ * in from the command line.
+ *
+ * Returns: TRUE if auto-branching is disabled.
+ **/
+gboolean
+gok_main_get_disable_autobranch (void)
+{
+	return gok_args.disable_autobranch;
+}
+
 /**
 * gok_main_get_inputdevice_name
 * 
@@ -3149,7 +3229,7 @@ gboolean
 gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback okfunc, gboolean opt_out)
 {
 
-	if (always || gok_scanner_current_state_uses_corepointer ()) 
+	if (!gok_main_get_use_corepointer() && ( always || gok_scanner_current_state_uses_corepointer() )) 
 	{
 		GtkWidget*	button;
 
@@ -3162,7 +3242,7 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_WARNING, 
 				GTK_BUTTONS_NONE,			
-				message);
+				"%s", message);
 			
 			button = gtk_button_new_from_stock(GTK_STOCK_HELP);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
