@@ -93,7 +93,10 @@ gchar* gok_main_get_access_method_override (void);
 gdouble gok_main_get_valuatorsensitivity_override (void);
 gboolean gok_main_get_extras (void);
 gboolean gok_main_get_login (void);
+gboolean gok_main_get_use_corepointer (void);
+gboolean gok_main_get_use_xevie (void);
 gchar* gok_main_get_inputdevice_name (void);
+gboolean gok_main_get_disable_autobranch (void);
 void gok_main_check_accessibility (void);
 void gok_main_warn_corepointer (gboolean always, gboolean configuring, gboolean opt_out);
 gboolean gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback okfunc, gboolean opt_out);
